<?php

declare(strict_types=1);

namespace FiloBlu\RuleDayFromBirthday\Block\Adminhtml\Promo\Quote\Edit\Tab\Coupons;

use Exception;
use Magento\Backend\Block\Template\Context;
use Magento\Backend\Block\Widget\Grid\Extended;
use Magento\Backend\Helper\Data;
use Magento\Framework\Exception\FileSystemException;
use Magento\Framework\Registry;
use Magento\SalesRule\Block\Adminhtml\Promo\Quote\Edit\Tab\Coupons\Grid\Column\Renderer\Used;
use Magento\SalesRule\Model\RegistryConstants;
use Magento\SalesRule\Model\ResourceModel\Coupon\Collection;
use Magento\SalesRule\Model\ResourceModel\Coupon\CollectionFactory;

/**
 * Coupon codes grid
 *
 * @api
 * @since 100.0.2
 */
class Grid extends Extended
{

    /**
     * Core registry
     *
     * @var Registry
     */
    protected $_coreRegistry = null;

    /**
     * @var CollectionFactory
     */
    protected $_salesRuleCoupon;

    /**
     * @param Context $context
     * @param Data $backendHelper
     * @param CollectionFactory $salesRuleCoupon
     * @param Registry $coreRegistry
     * @param array $data
     */
    public function __construct(
        Context           $context,
        Data              $backendHelper,
        CollectionFactory $salesRuleCoupon,
        Registry          $coreRegistry,
        array             $data = []
    )
    {
        $this->_coreRegistry = $coreRegistry;
        $this->_salesRuleCoupon = $salesRuleCoupon;
        parent::__construct($context, $backendHelper, $data);
    }

    /**
     * Get grid url
     *
     * @return string
     */
    public function getGridUrl()
    {
        return $this->getUrl('sales_rule/*/couponsGrid', ['_current' => true]);
    }

    /**
     * Constructor
     *
     * @return void
     * @throws FileSystemException
     */
    protected function _construct()
    {
        $this->setId('couponCodesGrid');
        $this->setUseAjax(true);
        parent::_construct();
    }

    /**
     * Prepare collection for grid
     *
     * @return $this
     */
    protected function _prepareCollection(): Grid
    {
        $priceRule = $this->_coreRegistry->registry(RegistryConstants::CURRENT_SALES_RULE);

        /** @var Collection $collection */
        $collection = $this->_salesRuleCoupon->create()->addRuleToFilter($priceRule)->addGeneratedCouponsFilter();
        $this->setCollection($collection);

        return parent::_prepareCollection();
    }

    /**
     * Define grid columns
     *
     * @return $this
     * @throws Exception
     */
    protected function _prepareColumns(): Grid
    {
        $this->addColumn('code', ['header' => __('Coupon Code'), 'index' => 'code']);

        $this->addColumn(
            'created_at', [
                'header' => __('Created'),
                'index'  => 'created_at',
                'type'   => 'datetime',
                'align'  => 'center',
                'width'  => '160'
            ]
        );
        $this->addColumn(
            'expiration_date', [
                'header' => __('Expiration Date'),
                'index'  => 'expiration_date',
                'type'   => 'datetime',
                'align'  => 'center',
                'width'  => '160'
            ]
        );

        $this->addColumn(
            'used', [
                'header'                    => __('Uses'),
                'index'                     => 'times_used',
                'width'                     => '100',
                'type'                      => 'options',
                'options'                   => [__('No'), __('Yes')],
                'renderer'                  => Used::class,
                'filter_condition_callback' => [$this->_salesRuleCoupon->create(), 'addIsUsedFilterCallback']
            ]
        );

        $this->addColumn(
            'times_used', ['header' => __('Times Used'), 'index' => 'times_used', 'width' => '50', 'type' => 'number']
        );

        $this->addExportType('*/*/exportCouponsCsv', __('CSV'));
        $this->addExportType('*/*/exportCouponsXml', __('Excel XML'));

        return parent::_prepareColumns();
    }

    /**
     * Configure grid mass actions
     *
     * @return $this
     */
    protected function _prepareMassaction(): Grid
    {
        $this->setMassactionIdField('coupon_id');
        $this->getMassactionBlock()->setFormFieldName('ids');
        $this->getMassactionBlock()->setUseAjax(true);
        $this->getMassactionBlock()->setHideFormElement(true);

        $this->getMassactionBlock()->addItem(
            'delete', [
                'label'    => __('Delete'),
                'url'      => $this->getUrl('sales_rule/*/couponsMassDelete', ['_current' => true]),
                'confirm'  => __('Are you sure you want to delete the selected coupon(s)?'),
                'complete' => 'refreshCouponCodesGrid'
            ]
        );

        return $this;
    }
}
