<?php

declare(strict_types=1);

namespace FiloBlu\RuleDayFromBirthday\Cron;

use Exception;
use FiloBlu\RuleDayFromBirthday\Helper\Data as Helper;
use FiloBlu\RuleDayFromBirthday\Controller\Adminhtml\Promo\Quote\Birthdaycoupon;
use Magento\Framework\Event\ManagerInterface;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Stdlib\DateTime\DateTime;
use RuntimeException;

/**
 * @class SendBulkAbstract
 * @package FiloBlu\BirthdayCoupon\Cron
 */
class SendBulkAbstract
{

    /**
     * @var Helper
     */
    protected $helper;
    /**
     * @var Birthdaycoupon
     */
    protected $birthdaycouponGenerator;
    /**
     * @var DateTime
     */
    protected $dateTime;
    /**
     * @var ManagerInterface
     */
    protected $eventManager;

    /**
     * @param Helper $helper
     * @param Birthdaycoupon $birthdaycouponGenerator
     * @param ManagerInterface $eventManager
     * @param DateTime $dateTime
     * @param array $entities
     */
    public function __construct(
        Helper           $helper,
        Birthdaycoupon   $birthdaycouponGenerator,
        ManagerInterface $eventManager,
        DateTime         $dateTime,
        array $entities = []
    ) {
        $this->helper = $helper;
        $this->birthdaycouponGenerator = $birthdaycouponGenerator;
        $this->eventManager = $eventManager;
        $this->dateTime = $dateTime;
    }

    /**
     * @throws LocalizedException
     * @throws InputException
     * @throws Exception
     */
    public function sendBulk(array $contactsArray, string $type): SendBulkAbstract
    {
        $storeContacts = [];
        foreach ($contactsArray as $contact) {
            $storeContacts[(int) $contact['store_id']][] = $contact;
        }

        $coupons = [];
        foreach ($storeContacts as $storeId => $contacts) {
            if (!$rule = $this->helper->getBirthdayCouponRule($storeId)) {
                unset($storeContacts[$storeId]);
                continue;
            }

            $couponQuantity = $this->countArray($contacts);
            $coupons[$storeId] = $this->helper->generateCoupon($rule, $couponQuantity);
        }

        foreach ($storeContacts as $storeId => $contacts) {
            if (!array_key_exists($storeId, $coupons)) {
                continue;
            }

            if (count($contacts) > count($coupons[$storeId])) {
                throw new RuntimeException(__METHOD__ . "Not enough coupons were created for store Id $storeId's contacts");
            }

            $bulkData = [];
            foreach ($contacts as $contact) {
                $bulkData[] = [
                    'contact' => $contact,
                    'coupon' => array_pop($coupons[$storeId])
                ];
            }

            $chunks = array_chunk($bulkData, $this->helper->maxSubscriberForBatchInBulk());
            foreach ($chunks as $chunk) {
                $this->eventManager->dispatch(
                    'filoblu_birthday_coupon_generate_after',
                    [
                        'bulkData' => $chunk,
                        'type' => $type
                    ]
                );
            }
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getBirthdayDate(): string
    {
        $daysOffset = $this->helper->getCouponCreationDaysOffset();

        return  $this->dateTime->date('Y-m-d H:i:s', strtotime("+ $daysOffset days"));
    }

    /**
     * @param array $array
     * @return int
     */
    private function countArray(array $array): int
    {
        if (!is_countable($array)) {
            return 0;
        }

        return count($array);
    }
}
